/*! \file noMac.c
 \brief No Medium Access Control Workshop MAC.
 
 @version 13
 @author Chris Hunter
 
 The easiest MAC. Forwards all traffic from
 ethernet to the radio medium and vice versa.
 No address filtering... no retransmissions.
 Medium access is completely dependent on
 the source traffic, so there is no actual
 medium access control. Hence, we have noMac.
 
 Note to workshop users: any names in quotes
 are names that can be searched for in the
 WARP API.
 
*/


#include "xparameters.h"
#include "warpmac.h"
#include "warpphy.h"
#include "noMac.h"
#include "string.h"
#include "errno.h"
#include "stdlib.h"
#include "stdio.h"

///Buffer to hold received packet
Macframe rxBuffer;

unsigned char pktBuf_rx;
unsigned char pktBuf_tx;

///@brief Callback for the reception of Ethernet packets
///
///This function is called by the ethernet MAC drivers
///when a packet is available to send. This function fills
///the Macframe transmit buffer with the packet and sends
///it over the OFDM link
///@param length Length, in bytes, of received Ethernet frame
///@param payload address of first byte in Ethernet payload.
void dataFromNetworkLayer_callback(Xuint32 length, char* payload)
{

//Pseudocode for workshop users:
//1) Instantiate a "Macframe" to hold a packet header
//2) Copy the "length" of the received Ethernet frame (argument of this function) into the appropriate field of the "phyHeader" in the "Macframe"
//3) Set the "fullRate" field of the "phyHeader" in the "Macframe" to "QPSK"
//4) Set the "codeRate" field of the "phyHeader" in the "Macframe" to "CODE_RATE_34" (a rate 3/4 convolutional code)
//5) Copy the header over the packet buffer 1 using the "warpmac_prepPhyForXmit" function.
//6) Send the contents of packet buffer 1 by using the "warpmac_startPhyXmit" function.
//7) Wait for the PHY transmitter to finish and enable the receiver by using the "warpmac_finishPhyXmit" function.

//Note: You may be wondering why the act of transmitting is broken up into three function calls (5,6,7 of pseudocode above).
//In this simple exercise, it seems a bit unnecessary. However, the reason for this division will become clear in later labs.

/**********************DELETE FOR WORKSHOP***************************/
	//Buffer for holding a packet-to-xmit
	Macframe txBuffer;
	//Set the length field in the header
	txBuffer.header.length = length;
	//Set the modulation scheme for the packet's full-rate symbols
	txBuffer.header.fullRate = HDR_FULLRATE_QPSK;
	//Set the payload coding rate
	txBuffer.header.codeRate = CODE_RATE_NONE;
	//Copy the header over to packet buffer 1
	warpmac_prepPhyForXmit(&txBuffer, 1);
	//Send packet buffer 1
	warpmac_startPhyXmit(1);
	//Wait for it to finish and enable the receiver
	warpmac_finishPhyXmit();
/**********************DELETE FOR WORKSHOP***************************/

}

///@brief Callback for the reception of bad wireless headers
///
///@param packet Pointer to received Macframe
void phyRx_badHeader_callback()
{
	warpmac_incrementLEDLow();
}

///@brief Callback for the reception of good wireless headers
///
///This function then polls the PHY to determine if the entire packet passes checksum
///thereby triggering the transmission of the received data over Ethernet.
///@param packet Pointer to received Macframe
void phyRx_goodHeader_callback(Macframe* packet)
{

//Pseudocode for workshop users:
//1) Instantiate a variable to keep track of the receiver's state. Default this state variable to "INCOMPLETE"
//2) Poll the state of the state of the receiver using "warpphy_pollRxStatus." Block until the state turns to either "GOODPACKET" or "BADPACKET"
//If "GOODPACKET"
//		3) Animate the top two LEDs to visualize this behavior using the "warpmac_incrementLEDHigh" function
//		4) Copy the received "Macframe" to Ethernet using "warpmac_prepEmacForXmit"	
//		5) Start the Emac using "warpmac_startEmacXmit"
//If "BADPACKET"
//		3) Animate the bottom two LEDs to visualize this behavior using the "warpmac_incrementLEDLow" function

	
/**********************DELETE FOR WORKSHOP***************************/	
	unsigned char state = PHYRXSTATUS_INCOMPLETE;	

	//Blocks until the PHY declares the payload good or bad
	state = warpmac_finishPhyRecv();
	
	if(state & PHYRXSTATUS_GOOD)
	{
		//Toggle the top user LEDs
		warpmac_incrementLEDHigh();

		//Starts the DMA transfer of the payload into the EMAC
		warpmac_prepPktToNetwork((void *)warpphy_getBuffAddr(pktBuf_rx)+NUM_HEADER_BYTES, (packet->header.length));

		//Waits until the DMA transfer is complete, then starts the EMAC
		warpmac_startPktToNetwork(packet->header.length);
	}
	
	if(state & PHYRXSTATUS_BAD)
	{
		//Toggle the bottom user LEDs
		warpmac_incrementLEDLow();
	}
/**********************DELETE FOR WORKSHOP***************************/
}

///@brief Main function
///
///This function configures MAC parameters, enables the underlying frameworks, and then loops forever.
int main()
{		
	xil_printf("NOMAC v13\r\n");

	pktBuf_rx = 0;
	pktBuf_tx = 1;
	
	//Initialize the framework
	// This function sets safe defaults for many parameters in the MAC/PHY frameworks
	// Many of these can be changed with other warpmac_ and warpphy_ calls
	//  or by customizing the warpmac.c/warpphy.c source
	warpmac_init();
	
	//Set the PHY for SISO using just the radio in slot 2
	warpphy_setAntennaMode(TX_ANTMODE_SISO_ANTA, RX_ANTMODE_SISO_ANTA);

	//Rx buffer is where the EMAC will DMA Wireless payloads from
	warpmac_setRxBuffers(&rxBuffer, pktBuf_rx);
	
	//Tx buffer is where the EMAC will DMA Ethernet payloads to
	warpmac_setPHYTxBuffer(pktBuf_tx);

	//Connect the various user-level callbacks
	warpmac_setCallback(EVENT_DATAFROMNETWORK, (void *)dataFromNetworkLayer_callback);
	warpmac_setCallback(EVENT_PHYGOODHEADER, (void *)phyRx_goodHeader_callback);
	warpmac_setCallback(EVENT_PHYBADHEADER, (void *)phyRx_badHeader_callback);

	//Set the default center frequency
	warpphy_setChannel(GHZ_2, 9);

	//Enable Ethernet
	warpmac_enableDataFromNetwork();
	
	while(1)
	{
		//Poll the timer, PHY and user I/O forever; actual processing will happen via callbacks above
		warpmac_pollPeripherals();
	}
	
	return;
}
